/*
 * @(#)RTPPacket.java
 * Created: 2005-04-21
 * Version: 2-0-alpha
 * Copyright (c) 2005-2006, University of Manchester All rights reserved. 
 * Andrew G D Rowley
 * Christian Vincenot <sipcom@cyberspace7.net>
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer. Redistributions in binary
 * form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials 
 * provided with the distribution. Neither the name of the University of 
 * Manchester nor the names of its contributors may be used to endorse or 
 * promote products derived from this software without specific prior written
 * permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package net.sf.fmj.media.rtp;

import java.io.IOException;
import java.net.DatagramPacket;

/**
 * Represents an RTP Packet
 * 
 * @author Andrew G D Rowley
 * @author Christian Vincenot
 * @version 1-1-alpha
 */
public class RTPPacket {
    // The header of the packet
    private RTPHeader header = null;

    // The packet data
    private byte[] data = null;

    /**
     * Creates a new RTPPacket
     * @param packet The datagram to get the packet from
     * @throws IOException I/O Exception
     */
    public RTPPacket(DatagramPacket packet) throws IOException {
        this(packet.getData(), packet.getOffset(), packet.getLength());
    }


    /**
     * Creates a new RTPPacket
     * @param offset the offset after which the RTP packet starts
     * @param length the total length
     * @param data The datagram to get the packet from
     * @throws IOException I/O Exception
     */
    public RTPPacket(byte[] data, int offset, int length) throws IOException {
        header = new RTPHeader(data, offset, length);
        this.data = new byte[length - RTPHeader.SIZE];
        System.arraycopy(data, offset + RTPHeader.SIZE, this.data, 0, length
                - RTPHeader.SIZE);
    }

    /**
     * Returns the RTP Header of this packet.
     * @return The header of the packet
     */
    public RTPHeader getRtpHeader() {
        return header;
    }

    /**
     * Returns the data contained in this packet.
     * @return The packet data
     */
    public byte[] getRtpData() {
        return data;
    }

    /**
     * Returns the length of this packet.
     * @return The length of the data
     */
    public int getRtpLength() {
        return data.length;
    }
}