/*
 * @(#)Movie.java 
 * 
 * Copyright (c) 2011 Werner Randelshofer, Goldau, Switzerland.
 * All rights reserved.
 * 
 * You may not use, copy or modify this file, except in compliance with the
 * license agreement you entered into with Werner Randelshofer.
 * For details see accompanying license terms.
 */
package org.monte.media;

import org.monte.media.math.Rational;
import java.beans.PropertyChangeListener;
import java.net.URI;

/**
 * A {@code movie} provides an editable container for media data in a time and
 * tracks coordinate system.
 *
 * @author Werner Randelshofer
 * @version $Id: Movie.java 320 2013-04-09 10:35:12Z werner $
 */
public interface Movie {

    public final static String INSERTION_POINT_PROPERTY = "insertionPoint";
    public final static String SELECTION_START_PROPERTY = "selectionStart";
    public final static String SELECTION_END_PROPERTY = "selectionEnd";
    public final static String URI_PROPERTY = "uri";

    /** Returns the total duration of the movie in seconds. */
    public Rational getDuration();

    /** Sets the position of the insertion point in seconds. */
    public void setInsertionPoint(Rational seconds);

    /** Returns the position of the insertion point in seconds.
     * If a movie has a duration of n, then there are n+1 insertion points.
     */
    public Rational getInsertionPoint();

    /** Returns the position of the in point in seconds. */
    public Rational getSelectionStart();

    /** Sets the position of the in point in seconds. */
    public void setSelectionStart(Rational in);

    /** Returns the position of the out point in seconds. */
    public Rational getSelectionEnd();

    /** Sets the position of the out point in seconds. */
    public void setSelectionEnd(Rational out);

    /** Returns the frame number for the specified time in seconds. */
    public long timeToSample(int track, Rational seconds);

    public void addPropertyChangeListener(PropertyChangeListener listener);

    public void removePropertyChangeListener(PropertyChangeListener listener);

    public Rational sampleToTime(int track, long sample);

    public int getTrackCount();

    public Format getFormat(int track);

    public Format getFileFormat();

    public MovieReader getReader();
    
    /** Returns the URI associated with this movie. This is typically a file,
     * but can be null if the movie only resides in memory. 
     * 
     * @return The Movie URI.
     */
    public URI getURI();
}
